@@ -2,8 +2,10 @@ package account
 
 import (
 	"errors"
+
+	"git.sr.ht/~rjarry/aerc/lib/statusline"
 	"git.sr.ht/~rjarry/aerc/widgets"
-	"time"
+	"git.sr.ht/~sircmpwn/getopt"
 )
 
 type Clear struct{}
@@ -29,7 +31,31 @@ func (Clear) Execute(aerc *widgets.Aerc, args []string) error {
 	if store == nil {
 		return errors.New("Cannot perform action. Messages still loading")
 	}
+
+	clearSelected := false
+	opts, optind, err := getopt.Getopts(args, "s")
+	if err != nil {
+		return err
+	}
+
+	for _, opt := range opts {
+		switch opt.Option {
+		case 's':
+			clearSelected = true
+		}
+	}
+
+	if len(args) != optind {
+		return errors.New("Usage: clear [-s]")
+	}
+
+	if clearSelected {
+		defer store.Select(0)
+	} else {
+		store.SetReselect(store.Selected())
+	}
 	store.ApplyClear()
-	aerc.PushStatus("Clear complete.", 10*time.Second)
+	acct.SetStatus(statusline.SearchFilterClear())
+
 	return nil
 }
