@@ -6,8 +6,9 @@ import (
 	"io/ioutil"
 	"os"
 	"path/filepath"
-	"strings"
 
+	"git.sr.ht/~rjarry/aerc/commands"
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~sircmpwn/getopt"
@@ -29,19 +30,34 @@ func (Recover) Complete(aerc *widgets.Aerc, args []string) []string {
 		filepath.Join(os.TempDir(), "aerc-compose-*.eml"),
 	)
 	if err != nil {
-		return []string{}
+		return make([]string, 0)
 	}
-	arg := strings.Join(args, " ")
-	if arg != "" {
-		for i, file := range files {
-			files[i] = strings.Join([]string{arg, file}, " ")
+	// if nothing is entered yet, return all files
+	if len(args) == 0 {
+		return files
+	}
+	if args[0] == "-" {
+		return []string{"-f"}
+	} else if args[0] == "-f" {
+		if len(args) == 1 {
+			for i, file := range files {
+				files[i] = args[0] + " " + file
+			}
+			return files
+		} else {
+			// only accepts one file to recover
+			return commands.FilterList(files, args[1], args[0]+" ",
+				aerc.SelectedAccountUiConfig().FuzzyComplete)
 		}
+	} else {
+		// only accepts one file to recover
+		return commands.FilterList(files, args[0], "", aerc.SelectedAccountUiConfig().FuzzyComplete)
 	}
-	return files
 }
 
 func (Recover) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(Recover{}.Complete(aerc, args)) == 0 {
+	// Complete() expects to be passed only the arguments, not including the command name
+	if len(Recover{}.Complete(aerc, args[1:])) == 0 {
 		return errors.New("No messages to recover.")
 	}
 
@@ -96,7 +112,11 @@ func (Recover) Execute(aerc *widgets.Aerc, args []string) error {
 		tab.Name = subject
 		tab.Content.Invalidate()
 	})
-	go composer.AppendContents(bytes.NewReader(data))
+	go func() {
+		defer logging.PanicHandler()
+
+		composer.AppendContents(bytes.NewReader(data))
+	}()
 
 	// remove file if force flag is set
 	if force {
