@@ -2,10 +2,11 @@ package account
 
 import (
 	"errors"
-	"fmt"
-	"time"
+	"strings"
 
+	"git.sr.ht/~rjarry/aerc/lib/statusline"
 	"git.sr.ht/~rjarry/aerc/widgets"
+	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 type SearchFilter struct{}
@@ -32,24 +33,29 @@ func (SearchFilter) Execute(aerc *widgets.Aerc, args []string) error {
 		return errors.New("Cannot perform action. Messages still loading")
 	}
 
-	var cb func([]uint32)
 	if args[0] == "filter" {
-		aerc.PushStatus("Filtering...", 10*time.Second)
-		cb = func(uids []uint32) {
-			aerc.PushStatus(fmt.Sprintf("Filter complete %s", args), 20*time.Second)
-			acct.Logger().Printf("Filter results: %v", uids)
-			store.ApplyFilter(uids)
+		if len(args[1:]) == 0 {
+			return Clear{}.Execute(aerc, []string{"clear"})
 		}
+		acct.SetStatus(statusline.FilterActivity("Filtering..."), statusline.Search(""))
+		store.SetFilter(args[1:])
+		cb := func(msg types.WorkerMessage) {
+			if _, ok := msg.(*types.Done); ok {
+				acct.SetStatus(statusline.FilterResult(strings.Join(args, " ")))
+				acct.Logger().Printf("Filter results: %v", store.Uids())
+			}
+		}
+		store.Sort(nil, cb)
 	} else {
-		aerc.PushStatus("Searching...", 10*time.Second)
-		cb = func(uids []uint32) {
-			aerc.PushStatus(fmt.Sprintf("Search complete %s", args), 20*time.Second)
+		acct.SetStatus(statusline.Search("Searching..."))
+		cb := func(uids []uint32) {
+			acct.SetStatus(statusline.Search(strings.Join(args, " ")))
 			acct.Logger().Printf("Search results: %v", uids)
 			store.ApplySearch(uids)
 			// TODO: Remove when stores have multiple OnUpdate handlers
 			acct.Messages().Invalidate()
 		}
+		store.Search(args, cb)
 	}
-	store.Search(args, cb)
 	return nil
 }
