@@ -1,13 +1,13 @@
 package compose
 
 import (
-	"io"
-	"io/ioutil"
+	"bytes"
 	"time"
 
 	"github.com/miolini/datacounter"
 	"github.com/pkg/errors"
 
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
@@ -31,6 +31,10 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 	if len(args) != 1 {
 		return errors.New("Usage: postpone")
 	}
+	acct := aerc.SelectedAccount()
+	if acct == nil {
+		return errors.New("No account selected")
+	}
 	composer, _ := aerc.SelectedTab().(*widgets.Composer)
 	config := composer.Config()
 	tabName := aerc.TabNames()[aerc.SelectedTabIndex()]
@@ -48,7 +52,7 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 	header.SetContentType("text/plain", map[string]string{"charset": "UTF-8"})
 	header.Set("Content-Transfer-Encoding", "quoted-printable")
 	worker := composer.Worker()
-	dirs := aerc.SelectedAccount().Directories().List()
+	dirs := acct.Directories().List()
 	alreadyCreated := false
 	for _, dir := range dirs {
 		if dir == config.Postpone {
@@ -62,6 +66,8 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 	// run this as a goroutine so we can make other progress. The message
 	// will be saved once the directory is created.
 	go func() {
+		defer logging.PanicHandler()
+
 		errStr := <-errChan
 		if errStr != "" {
 			aerc.PushError(errStr)
@@ -75,33 +81,29 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 		}
 
 		aerc.RemoveTab(composer)
-		ctr := datacounter.NewWriterCounter(ioutil.Discard)
+		var buf bytes.Buffer
+		ctr := datacounter.NewWriterCounter(&buf)
 		err = composer.WriteMessage(header, ctr)
 		if err != nil {
 			handleErr(errors.Wrap(err, "WriteMessage"))
 			return
 		}
 		nbytes := int(ctr.Count())
-		r, w := io.Pipe()
 		worker.PostAction(&types.AppendMessage{
 			Destination: config.Postpone,
 			Flags:       []models.Flag{models.SeenFlag},
 			Date:        time.Now(),
-			Reader:      r,
+			Reader:      &buf,
 			Length:      int(nbytes),
 		}, func(msg types.WorkerMessage) {
 			switch msg := msg.(type) {
 			case *types.Done:
 				aerc.PushStatus("Message postponed.", 10*time.Second)
-				r.Close()
 				composer.Close()
 			case *types.Error:
-				r.Close()
 				handleErr(msg.Error)
 			}
 		})
-		composer.WriteMessage(header, w)
-		w.Close()
 	}()
 
 	if !alreadyCreated {
