@@ -24,13 +24,7 @@ func (ChangeTab) Complete(aerc *widgets.Aerc, args []string) []string {
 		return aerc.TabNames()
 	}
 	joinedArgs := strings.Join(args, " ")
-	out := make([]string, 0)
-	for _, tab := range aerc.TabNames() {
-		if strings.HasPrefix(tab, joinedArgs) {
-			out = append(out, tab)
-		}
-	}
-	return out
+	return FilterList(aerc.TabNames(), joinedArgs, "", aerc.SelectedAccountUiConfig().FuzzyComplete)
 }
 
 func (ChangeTab) Execute(aerc *widgets.Aerc, args []string) error {
