@@ -21,25 +21,30 @@ type MessageStore struct {
 	Threads []*types.Thread
 
 	selected        int
+	reselect        *models.MessageInfo
 	bodyCallbacks   map[uint32][]func(*types.FullMessage)
 	headerCallbacks map[uint32][]func(*types.MessageInfo)
 
 	//marking
 	marked         map[uint32]struct{}
+	lastMarked     map[uint32]struct{}
 	visualStartUid uint32
 	visualMarkMode bool
 
 	// Search/filter results
 	results     []uint32
 	resultIndex int
-	filter      bool
+	filter      []string
 
-	defaultSortCriteria []*types.SortCriterion
+	sortCriteria []*types.SortCriterion
 
-	thread bool
+	threadedView bool
+	buildThreads bool
+	builder      *ThreadBuilder
 
 	// Map of uids we've asked the worker to fetch
 	onUpdate       func(store *MessageStore) // TODO: multiple onUpdate handlers
+	onFilterChange func(store *MessageStore)
 	onUpdateDirs   func()
 	pendingBodies  map[uint32]interface{}
 	pendingHeaders map[uint32]interface{}
@@ -55,12 +60,16 @@ type MessageStore struct {
 func NewMessageStore(worker *types.Worker,
 	dirInfo *models.DirectoryInfo,
 	defaultSortCriteria []*types.SortCriterion,
-	thread bool,
+	thread bool, clientThreads bool,
 	triggerNewEmail func(*models.MessageInfo),
 	triggerDirectoryChange func()) *MessageStore {
 
 	dirInfoUpdateDelay := 5 * time.Second
 
+	if !dirInfo.Caps.Thread {
+		clientThreads = true
+	}
+
 	return &MessageStore{
 		Deleted:  make(map[uint32]interface{}),
 		DirInfo:  *dirInfo,
@@ -71,9 +80,11 @@ func NewMessageStore(worker *types.Worker,
 		bodyCallbacks:   make(map[uint32][]func(*types.FullMessage)),
 		headerCallbacks: make(map[uint32][]func(*types.MessageInfo)),
 
-		thread: thread,
+		threadedView: thread,
+		buildThreads: clientThreads,
 
-		defaultSortCriteria: defaultSortCriteria,
+		filter:       []string{"filter"},
+		sortCriteria: defaultSortCriteria,
 
 		pendingBodies:  make(map[uint32]interface{}),
 		pendingHeaders: make(map[uint32]interface{}),
@@ -107,7 +118,15 @@ func (store *MessageStore) FetchHeaders(uids []uint32,
 		}
 	}
 	if len(toFetch) > 0 {
-		store.worker.PostAction(&types.FetchMessageHeaders{Uids: toFetch}, nil)
+		store.worker.PostAction(&types.FetchMessageHeaders{Uids: toFetch}, func(msg types.WorkerMessage) {
+			switch msg.(type) {
+			case *types.Error:
+				for _, uid := range toFetch {
+					delete(store.pendingHeaders, uid)
+					delete(store.headerCallbacks, uid)
+				}
+			}
+		})
 	}
 }
 
@@ -135,6 +154,7 @@ func (store *MessageStore) FetchFull(uids []uint32, cb func(*types.FullMessage))
 			switch msg.(type) {
 			case *types.Error:
 				for _, uid := range toFetch {
+					delete(store.pendingBodies, uid)
 					delete(store.bodyCallbacks, uid)
 				}
 			}
@@ -177,10 +197,15 @@ func merge(to *models.MessageInfo, from *models.MessageInfo) {
 func (store *MessageStore) Update(msg types.WorkerMessage) {
 	update := false
 	directoryChange := false
+	if store.reselect == nil {
+		store.SetReselect(store.Selected())
+	}
 	switch msg := msg.(type) {
 	case *types.DirectoryInfo:
 		store.DirInfo = *msg.Info
-		store.Sort(store.defaultSortCriteria, nil)
+		if !msg.SkipSort {
+			store.Sort(store.sortCriteria, nil)
+		}
 		update = true
 	case *types.DirectoryContents:
 		newMap := make(map[uint32]*models.MessageInfo)
@@ -194,6 +219,7 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		}
 		store.Messages = newMap
 		store.uids = msg.Uids
+		store.checkMark()
 		update = true
 	case *types.DirectoryThreaded:
 		var uids []uint32
@@ -214,13 +240,16 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		}
 		store.Messages = newMap
 		store.uids = uids
+		store.checkMark()
 		store.Threads = msg.Threads
 		update = true
 	case *types.MessageInfo:
 		if existing, ok := store.Messages[msg.Info.Uid]; ok && existing != nil {
 			merge(existing, msg.Info)
 		} else {
-			store.Messages[msg.Info.Uid] = msg.Info
+			if msg.Info.Envelope != nil {
+				store.Messages[msg.Info.Uid] = msg.Info
+			}
 		}
 		seen := false
 		recent := false
@@ -242,6 +271,9 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 				}
 			}
 		}
+		if store.builder != nil {
+			store.builder.Update(msg.Info)
+		}
 		update = true
 	case *types.FullMessage:
 		if _, ok := store.pendingBodies[msg.Content.Uid]; ok {
@@ -309,6 +341,10 @@ func (store *MessageStore) OnUpdate(fn func(store *MessageStore)) {
 	store.onUpdate = fn
 }
 
+func (store *MessageStore) OnFilterChange(fn func(store *MessageStore)) {
+	store.onFilterChange = fn
+}
+
 func (store *MessageStore) OnUpdateDirs(fn func()) {
 	store.onUpdateDirs = fn
 }
@@ -320,6 +356,39 @@ func (store *MessageStore) update() {
 	if store.onUpdateDirs != nil {
 		store.onUpdateDirs()
 	}
+	if store.BuildThreads() {
+		store.runThreadBuilder()
+	}
+}
+
+func (store *MessageStore) SetThreadedView(thread bool) {
+	store.threadedView = thread
+	if store.buildThreads {
+		if store.threadedView {
+			store.runThreadBuilder()
+		}
+		store.Reselect()
+		return
+	}
+	store.Sort(store.sortCriteria, nil)
+}
+
+func (store *MessageStore) ThreadedView() bool {
+	return store.threadedView
+}
+
+func (store *MessageStore) BuildThreads() bool {
+	return store.buildThreads
+}
+
+func (store *MessageStore) runThreadBuilder() {
+	if store.builder == nil {
+		store.builder = NewThreadBuilder(store.worker.Logger)
+		for _, msg := range store.Messages {
+			store.builder.Update(msg)
+		}
+	}
+	store.Threads = store.builder.Threads(store.uids)
 }
 
 func (store *MessageStore) Delete(uids []uint32,
@@ -329,8 +398,22 @@ func (store *MessageStore) Delete(uids []uint32,
 		store.Deleted[uid] = nil
 	}
 
-	store.worker.PostAction(&types.DeleteMessages{Uids: uids}, cb)
-	store.update()
+	store.worker.PostAction(&types.DeleteMessages{Uids: uids},
+		func(msg types.WorkerMessage) {
+			switch msg.(type) {
+			case *types.Error:
+				store.revertDeleted(uids)
+			}
+			cb(msg)
+		})
+}
+
+func (store *MessageStore) revertDeleted(uids []uint32) {
+	for _, uid := range uids {
+		if _, ok := store.Deleted[uid]; ok {
+			delete(store.Deleted, uid)
+		}
+	}
 }
 
 func (store *MessageStore) Copy(uids []uint32, dest string, createDest bool,
@@ -369,13 +452,12 @@ func (store *MessageStore) Move(uids []uint32, dest string, createDest bool,
 	}, func(msg types.WorkerMessage) {
 		switch msg.(type) {
 		case *types.Error:
+			store.revertDeleted(uids)
 			cb(msg)
 		case *types.Done:
-			store.worker.PostAction(&types.DeleteMessages{Uids: uids}, cb)
+			store.Delete(uids, cb)
 		}
 	})
-
-	store.update()
 }
 
 func (store *MessageStore) Flag(uids []uint32, flag models.Flag,
@@ -398,14 +480,22 @@ func (store *MessageStore) Answered(uids []uint32, answered bool,
 }
 
 func (store *MessageStore) Uids() []uint32 {
-	if store.filter {
-		return store.results
+
+	if store.ThreadedView() && store.builder != nil {
+		if uids := store.builder.Uids(); len(uids) > 0 {
+			return uids
+		}
 	}
 	return store.uids
 }
 
 func (store *MessageStore) Selected() *models.MessageInfo {
-	return store.Messages[store.Uids()[len(store.Uids())-store.selected-1]]
+	uids := store.Uids()
+	idx := len(uids) - store.selected - 1
+	if len(uids) == 0 || idx < 0 || idx >= len(uids) {
+		return nil
+	}
+	return store.Messages[uids[idx]]
 }
 
 func (store *MessageStore) SelectedIndex() int {
@@ -423,6 +513,26 @@ func (store *MessageStore) Select(index int) {
 	store.updateVisual()
 }
 
+func (store *MessageStore) Reselect() {
+	if store.reselect == nil {
+		return
+	}
+	uid := store.reselect.Uid
+	newIdx := 0
+	for idx, uidStore := range store.Uids() {
+		if uidStore == uid {
+			newIdx = len(store.Uids()) - idx - 1
+			break
+		}
+	}
+	store.reselect = nil
+	store.Select(newIdx)
+}
+
+func (store *MessageStore) SetReselect(info *models.MessageInfo) {
+	store.reselect = info
+}
+
 // Mark sets the marked state on a MessageInfo
 func (store *MessageStore) Mark(uid uint32) {
 	if store.visualMarkMode {
@@ -442,6 +552,10 @@ func (store *MessageStore) Unmark(uid uint32) {
 	delete(store.marked, uid)
 }
 
+func (store *MessageStore) Remark() {
+	store.marked = store.lastMarked
+}
+
 // ToggleMark toggles the marked state on a MessageInfo
 func (store *MessageStore) ToggleMark(uid uint32) {
 	if store.visualMarkMode {
@@ -457,9 +571,26 @@ func (store *MessageStore) ToggleMark(uid uint32) {
 
 // resetMark removes the marking from all messages
 func (store *MessageStore) resetMark() {
+	store.lastMarked = store.marked
 	store.marked = make(map[uint32]struct{})
 }
 
+// checkMark checks that no stale uids remain marked
+func (store *MessageStore) checkMark() {
+	for mark := range store.marked {
+		present := false
+		for _, uid := range store.uids {
+			if mark == uid {
+				present = true
+				break
+			}
+		}
+		if !present {
+			delete(store.marked, mark)
+		}
+	}
+}
+
 //IsMarked checks whether a MessageInfo has been marked
 func (store *MessageStore) IsMarked(uid uint32) bool {
 	_, marked := store.marked[uid]
@@ -525,7 +656,7 @@ func (store *MessageStore) updateVisual() {
 	}
 	missing := make([]uint32, 0)
 	for _, uid := range visUids {
-		if msg, _ := store.Messages[uid]; msg == nil {
+		if msg := store.Messages[uid]; msg == nil {
 			missing = append(missing, uid)
 		}
 	}
@@ -570,8 +701,18 @@ func (store *MessageStore) Search(args []string, cb func([]uint32)) {
 	}, func(msg types.WorkerMessage) {
 		switch msg := msg.(type) {
 		case *types.SearchResults:
-			sort.SortBy(msg.Uids, store.uids)
-			cb(msg.Uids)
+			allowedUids := store.Uids()
+			uids := make([]uint32, 0, len(msg.Uids))
+			for _, uid := range msg.Uids {
+				for _, uidCheck := range allowedUids {
+					if uid == uidCheck {
+						uids = append(uids, uid)
+						break
+					}
+				}
+			}
+			sort.SortBy(uids, allowedUids)
+			cb(uids)
 		}
 	})
 }
@@ -582,18 +723,26 @@ func (store *MessageStore) ApplySearch(results []uint32) {
 	store.NextResult()
 }
 
-func (store *MessageStore) ApplyFilter(results []uint32) {
-	store.results = results
-	store.filter = true
-	store.update()
-	// any marking is now invalid
-	// TODO: could save that probably
-	store.ClearVisualMark()
+func (store *MessageStore) SetFilter(args []string) {
+	store.filter = append(store.filter, args...)
 }
 
 func (store *MessageStore) ApplyClear() {
+	if store.reselect == nil {
+		store.SetReselect(store.Selected())
+	}
+	store.filter = []string{"filter"}
 	store.results = nil
-	store.filter = false
+	if store.onFilterChange != nil {
+		store.onFilterChange(store)
+	}
+	cb := func(msg types.WorkerMessage) {
+		switch msg.(type) {
+		case *types.Done:
+			store.Reselect()
+		}
+	}
+	store.Sort(nil, cb)
 }
 
 func (store *MessageStore) nextPrevResult(delta int) {
@@ -607,9 +756,10 @@ func (store *MessageStore) nextPrevResult(delta int) {
 	if store.resultIndex < 0 {
 		store.resultIndex = len(store.results) - 1
 	}
-	for i, uid := range store.uids {
+	uids := store.Uids()
+	for i, uid := range uids {
 		if store.results[len(store.results)-store.resultIndex-1] == uid {
-			store.Select(len(store.uids) - i - 1)
+			store.Select(len(uids) - i - 1)
 			break
 		}
 	}
@@ -633,23 +783,30 @@ func (store *MessageStore) ModifyLabels(uids []uint32, add, remove []string,
 	}, cb)
 }
 
-func (store *MessageStore) Sort(criteria []*types.SortCriterion, cb func()) {
+func (store *MessageStore) Sort(criteria []*types.SortCriterion, cb func(types.WorkerMessage)) {
+	if criteria == nil {
+		criteria = store.sortCriteria
+	} else {
+		store.sortCriteria = criteria
+	}
 	store.Sorting = true
 
 	handle_return := func(msg types.WorkerMessage) {
 		store.Sorting = false
 		if cb != nil {
-			cb()
+			cb(msg)
 		}
 	}
 
-	if store.thread {
+	if store.threadedView && !store.buildThreads {
 		store.worker.PostAction(&types.FetchDirectoryThreaded{
-			SortCriteria: criteria,
+			SortCriteria:   criteria,
+			FilterCriteria: store.filter,
 		}, handle_return)
 	} else {
 		store.worker.PostAction(&types.FetchDirectoryContents{
-			SortCriteria: criteria,
+			SortCriteria:   criteria,
+			FilterCriteria: store.filter,
 		}, handle_return)
 	}
 }
