@@ -75,21 +75,21 @@ type AccountWizard struct {
 
 func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	wizard := &AccountWizard{
-		accountName:  ui.NewTextInput("", conf.Ui).Prompt("> "),
+		accountName:  ui.NewTextInput("", &conf.Ui).Prompt("> "),
 		aerc:         aerc,
 		conf:         conf,
 		temporary:    false,
 		copySent:     true,
-		email:        ui.NewTextInput("", conf.Ui).Prompt("> "),
-		fullName:     ui.NewTextInput("", conf.Ui).Prompt("> "),
-		imapPassword: ui.NewTextInput("", conf.Ui).Prompt("] ").Password(true),
-		imapServer:   ui.NewTextInput("", conf.Ui).Prompt("> "),
+		email:        ui.NewTextInput("", &conf.Ui).Prompt("> "),
+		fullName:     ui.NewTextInput("", &conf.Ui).Prompt("> "),
+		imapPassword: ui.NewTextInput("", &conf.Ui).Prompt("] ").Password(true),
+		imapServer:   ui.NewTextInput("", &conf.Ui).Prompt("> "),
 		imapStr:      ui.NewText("imaps://", conf.Ui.GetStyle(config.STYLE_DEFAULT)),
-		imapUsername: ui.NewTextInput("", conf.Ui).Prompt("> "),
-		smtpPassword: ui.NewTextInput("", conf.Ui).Prompt("] ").Password(true),
-		smtpServer:   ui.NewTextInput("", conf.Ui).Prompt("> "),
+		imapUsername: ui.NewTextInput("", &conf.Ui).Prompt("> "),
+		smtpPassword: ui.NewTextInput("", &conf.Ui).Prompt("] ").Password(true),
+		smtpServer:   ui.NewTextInput("", &conf.Ui).Prompt("> "),
 		smtpStr:      ui.NewText("smtps://", conf.Ui.GetStyle(config.STYLE_DEFAULT)),
-		smtpUsername: ui.NewTextInput("", conf.Ui).Prompt("> "),
+		smtpUsername: ui.NewTextInput("", &conf.Ui).Prompt("> "),
 	}
 
 	// Autofill some stuff for the user
@@ -136,18 +136,18 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	})
 
 	basics := ui.NewGrid().Rows([]ui.GridSpec{
-		{ui.SIZE_EXACT, ui.Const(8)}, // Introduction
-		{ui.SIZE_EXACT, ui.Const(1)}, // Account name (label)
-		{ui.SIZE_EXACT, ui.Const(1)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Full name (label)
-		{ui.SIZE_EXACT, ui.Const(1)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Email address (label)
-		{ui.SIZE_EXACT, ui.Const(1)}, // (input)
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(8)}, // Introduction
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Account name (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Full name (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Email address (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // (input)
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	}).Columns([]ui.GridSpec{
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 	basics.AddChild(
 		ui.NewText("\nWelcome to aerc! Let's configure your account.\n\n"+
@@ -179,7 +179,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		At(7, 0)
 	basics.AddChild(wizard.email).
 		At(8, 0)
-	selector := NewSelector([]string{"Next"}, 0, conf.Ui).
+	selector := NewSelector([]string{"Next"}, 0, &conf.Ui).
 		OnChoose(func(option string) {
 			email := wizard.email.String()
 			if strings.ContainsRune(email, '@') {
@@ -212,23 +212,23 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	})
 
 	incoming := ui.NewGrid().Rows([]ui.GridSpec{
-		{ui.SIZE_EXACT, ui.Const(3)}, // Introduction
-		{ui.SIZE_EXACT, ui.Const(1)}, // Username (label)
-		{ui.SIZE_EXACT, ui.Const(1)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Password (label)
-		{ui.SIZE_EXACT, ui.Const(1)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Server (label)
-		{ui.SIZE_EXACT, ui.Const(1)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Connection mode (label)
-		{ui.SIZE_EXACT, ui.Const(2)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Connection string
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(3)}, // Introduction
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Username (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Password (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Server (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Connection mode (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(2)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Connection string
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	}).Columns([]ui.GridSpec{
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 	incoming.AddChild(ui.NewText("\nConfigure incoming mail (IMAP)",
 		conf.Ui.GetStyle(config.STYLE_DEFAULT)))
@@ -265,7 +265,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		"IMAP over SSL/TLS",
 		"IMAP with STARTTLS",
 		"Insecure IMAP",
-	}, 0, conf.Ui).Chooser(true).OnSelect(func(option string) {
+	}, 0, &conf.Ui).Chooser(true).OnSelect(func(option string) {
 		switch option {
 		case "IMAP over SSL/TLS":
 			wizard.imapMode = IMAP_OVER_TLS
@@ -277,7 +277,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		wizard.imapUri()
 	})
 	incoming.AddChild(imapMode).At(11, 0)
-	selector = NewSelector([]string{"Previous", "Next"}, 1, conf.Ui).
+	selector = NewSelector([]string{"Previous", "Next"}, 1, &conf.Ui).
 		OnChoose(wizard.advance)
 	incoming.AddChild(ui.NewFill(' ', tcell.StyleDefault)).At(12, 0)
 	incoming.AddChild(wizard.imapStr).At(13, 0)
@@ -291,26 +291,26 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	})
 
 	outgoing := ui.NewGrid().Rows([]ui.GridSpec{
-		{ui.SIZE_EXACT, ui.Const(3)}, // Introduction
-		{ui.SIZE_EXACT, ui.Const(1)}, // Username (label)
-		{ui.SIZE_EXACT, ui.Const(1)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Password (label)
-		{ui.SIZE_EXACT, ui.Const(1)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Server (label)
-		{ui.SIZE_EXACT, ui.Const(1)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Connection mode (label)
-		{ui.SIZE_EXACT, ui.Const(2)}, // (input)
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Connection string
-		{ui.SIZE_EXACT, ui.Const(1)}, // Padding
-		{ui.SIZE_EXACT, ui.Const(1)}, // Copy to sent (label)
-		{ui.SIZE_EXACT, ui.Const(2)}, // (input)
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(3)}, // Introduction
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Username (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Password (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Server (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Connection mode (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(2)}, // (input)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Connection string
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Padding
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)}, // Copy to sent (label)
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(2)}, // (input)
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	}).Columns([]ui.GridSpec{
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 	outgoing.AddChild(ui.NewText("\nConfigure outgoing mail (SMTP)",
 		conf.Ui.GetStyle(config.STYLE_DEFAULT)))
@@ -347,7 +347,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		"SMTP over SSL/TLS",
 		"SMTP with STARTTLS",
 		"Insecure SMTP",
-	}, 0, conf.Ui).Chooser(true).OnSelect(func(option string) {
+	}, 0, &conf.Ui).Chooser(true).OnSelect(func(option string) {
 		switch option {
 		case "SMTP over SSL/TLS":
 			wizard.smtpMode = SMTP_OVER_TLS
@@ -359,7 +359,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		wizard.smtpUri()
 	})
 	outgoing.AddChild(smtpMode).At(11, 0)
-	selector = NewSelector([]string{"Previous", "Next"}, 1, conf.Ui).
+	selector = NewSelector([]string{"Previous", "Next"}, 1, &conf.Ui).
 		OnChoose(wizard.advance)
 	outgoing.AddChild(ui.NewFill(' ', tcell.StyleDefault)).At(12, 0)
 	outgoing.AddChild(wizard.smtpStr).At(13, 0)
@@ -367,7 +367,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	outgoing.AddChild(
 		ui.NewText("Copy sent messages to 'Sent' folder?",
 			conf.Ui.GetStyle(config.STYLE_HEADER))).At(15, 0)
-	copySent := NewSelector([]string{"Yes", "No"}, 0, conf.Ui).
+	copySent := NewSelector([]string{"Yes", "No"}, 0, &conf.Ui).
 		Chooser(true).OnChoose(func(option string) {
 		switch option {
 		case "Yes":
@@ -387,10 +387,10 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	})
 
 	complete := ui.NewGrid().Rows([]ui.GridSpec{
-		{ui.SIZE_EXACT, ui.Const(7)},  // Introduction
-		{ui.SIZE_WEIGHT, ui.Const(1)}, // Previous / Finish / Finish & open tutorial
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(7)},  // Introduction
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)}, // Previous / Finish / Finish & open tutorial
 	}).Columns([]ui.GridSpec{
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 	complete.AddChild(ui.NewText(
 		"\nConfiguration complete!\n\n"+
@@ -402,7 +402,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		"Previous",
 		"Finish & open tutorial",
 		"Finish",
-	}, 1, conf.Ui).OnChoose(func(option string) {
+	}, 1, &conf.Ui).OnChoose(func(option string) {
 		switch option {
 		case "Previous":
 			wizard.advance("Previous")
@@ -535,7 +535,7 @@ func (wizard *AccountWizard) finish(tutorial bool) {
 	wizard.conf.Accounts = append(wizard.conf.Accounts, account)
 
 	view, err := NewAccountView(wizard.aerc, wizard.conf, &account,
-		wizard.aerc.logger, wizard.aerc)
+		wizard.aerc.logger, wizard.aerc, nil)
 	if err != nil {
 		wizard.aerc.NewTab(errorScreen(err.Error(), wizard.conf.Ui),
 			account.Name)
