@@ -1,23 +1,17 @@
 package widgets
 
 import (
-	"bufio"
 	"bytes"
 	"fmt"
 	"io"
 	"io/ioutil"
-	"mime"
-	"net/http"
 	"net/textproto"
 	"os"
 	"os/exec"
-	"path/filepath"
 	"strings"
 	"time"
 
-	"github.com/ProtonMail/go-crypto/openpgp"
 	"github.com/emersion/go-message/mail"
-	"github.com/emersion/go-pgpmail"
 	"github.com/gdamore/tcell/v2"
 	"github.com/mattn/go-runewidth"
 	"github.com/mitchellh/go-homedir"
@@ -43,7 +37,7 @@ type Composer struct {
 	acct       *AccountView
 	aerc       *Aerc
 
-	attachments []string
+	attachments []lib.Attachment
 	editor      *Terminal
 	email       *os.File
 	grid        *ui.Grid
@@ -51,8 +45,10 @@ type Composer struct {
 	review      *reviewMessage
 	worker      *types.Worker
 	completer   *completer.Completer
+	crypto      *cryptoStatus
 	sign        bool
 	encrypt     bool
+	attachKey   bool
 
 	layout    HeaderLayout
 	focusable []ui.MouseableDrawableInteractive
@@ -62,6 +58,8 @@ type Composer struct {
 	onClose []func(ti *Composer)
 
 	width int
+
+	textParts []*lib.Part
 }
 
 func NewComposer(aerc *Aerc, acct *AccountView, conf *config.AercConfig,
@@ -117,8 +115,16 @@ func NewComposer(aerc *Aerc, acct *AccountView, conf *config.AercConfig,
 	c.AddSignature()
 
 	c.updateGrid()
+	c.updateCrypto()
 	c.ShowTerminal()
 
+	if c.acctConfig.PgpAutoSign {
+		c.SetSign(true)
+	}
+	if c.acctConfig.PgpOpportunisticEncrypt {
+		c.SetEncrypt(true)
+	}
+
 	return c, nil
 }
 
@@ -127,15 +133,15 @@ func (c *Composer) buildComposeHeader(aerc *Aerc, cmpl *completer.Completer) {
 	c.layout = aerc.conf.Compose.HeaderLayout
 	c.editors = make(map[string]*headerEditor)
 	c.focusable = make([]ui.MouseableDrawableInteractive, 0)
+	uiConfig := c.acct.UiConfig()
 
 	for i, row := range c.layout {
 		for j, h := range row {
 			h = strings.ToLower(h)
 			c.layout[i][j] = h // normalize to lowercase
-			e := newHeaderEditor(h, c.header, aerc.SelectedAccount().UiConfig())
+			e := newHeaderEditor(h, c.header, uiConfig)
 			if aerc.conf.Ui.CompletionPopovers {
-				e.input.TabComplete(cmpl.ForHeader(h),
-					aerc.SelectedAccount().UiConfig().CompletionDelay)
+				e.input.TabComplete(cmpl.ForHeader(h), uiConfig.CompletionDelay)
 			}
 			c.editors[h] = e
 			switch h {
@@ -152,9 +158,9 @@ func (c *Composer) buildComposeHeader(aerc *Aerc, cmpl *completer.Completer) {
 	for _, h := range []string{"cc", "bcc"} {
 		if c.header.Has(h) {
 			if _, ok := c.editors[h]; !ok {
-				e := newHeaderEditor(h, c.header, aerc.SelectedAccount().UiConfig())
+				e := newHeaderEditor(h, c.header, uiConfig)
 				if aerc.conf.Ui.CompletionPopovers {
-					e.input.TabComplete(cmpl.ForHeader(h), aerc.SelectedAccount().UiConfig().CompletionDelay)
+					e.input.TabComplete(cmpl.ForHeader(h), uiConfig.CompletionDelay)
 				}
 				c.editors[h] = e
 				c.focusable = append(c.focusable, e)
@@ -177,9 +183,75 @@ func (c *Composer) Sent() bool {
 	return c.sent
 }
 
-func (c *Composer) SetSign(sign bool) *Composer {
+func (c *Composer) SetAttachKey(attach bool) error {
+	if !attach {
+		name := c.crypto.signKey + ".asc"
+		found := false
+		for _, a := range c.attachments {
+			if a.Name() == name {
+				found = true
+			}
+		}
+		if found {
+			c.DeleteAttachment(name)
+		} else {
+			attach = !attach
+		}
+	}
+	if attach {
+		var s string
+		var err error
+		if c.crypto.signKey == "" {
+			if c.acctConfig.PgpKeyId != "" {
+				s = c.acctConfig.PgpKeyId
+			} else {
+				s, err = getSenderEmail(c)
+				if err != nil {
+					return err
+				}
+			}
+			c.crypto.signKey, err = c.aerc.Crypto.GetSignerKeyId(s)
+			if err != nil {
+				return err
+			}
+		}
+
+		r, err := c.aerc.Crypto.ExportKey(c.crypto.signKey)
+		if err != nil {
+			return err
+		}
+
+		c.attachments = append(c.attachments,
+			lib.NewPartAttachment(
+				lib.NewPart(
+					"application/pgp-keys",
+					map[string]string{"charset": "UTF-8"},
+					r,
+				),
+				c.crypto.signKey+".asc",
+			),
+		)
+
+	}
+
+	c.attachKey = attach
+
+	c.resetReview()
+	return nil
+}
+
+func (c *Composer) AttachKey() bool {
+	return c.attachKey
+}
+
+func (c *Composer) SetSign(sign bool) error {
 	c.sign = sign
-	return c
+	err := c.updateCrypto()
+	if err != nil {
+		c.sign = !sign
+		return fmt.Errorf("Cannot sign message: %v", err)
+	}
+	return nil
 }
 
 func (c *Composer) Sign() bool {
@@ -187,7 +259,21 @@ func (c *Composer) Sign() bool {
 }
 
 func (c *Composer) SetEncrypt(encrypt bool) *Composer {
-	c.encrypt = encrypt
+	if !encrypt {
+		c.encrypt = encrypt
+		c.updateCrypto()
+		return c
+	}
+	// Check on any attempt to encrypt, and any lost focus of "to", "cc", or
+	// "bcc" field. Use OnFocusLost instead of OnChange to limit keyring checks
+	c.encrypt = c.checkEncryptionKeys("")
+	if c.crypto.setEncOneShot {
+		// Prevent registering a lot of callbacks
+		c.OnFocusLost("to", c.checkEncryptionKeys)
+		c.OnFocusLost("cc", c.checkEncryptionKeys)
+		c.OnFocusLost("bcc", c.checkEncryptionKeys)
+		c.crypto.setEncOneShot = false
+	}
 	return c
 }
 
@@ -195,6 +281,56 @@ func (c *Composer) Encrypt() bool {
 	return c.encrypt
 }
 
+func (c *Composer) updateCrypto() error {
+	if c.crypto == nil {
+		uiConfig := c.acct.UiConfig()
+		c.crypto = newCryptoStatus(uiConfig)
+	}
+	var err error
+	// Check if signKey is empty so we only run this once
+	if c.sign && c.crypto.signKey == "" {
+		cp := c.aerc.Crypto
+		var s string
+		if c.acctConfig.PgpKeyId != "" {
+			s = c.acctConfig.PgpKeyId
+		} else {
+			s, err = getSenderEmail(c)
+			if err != nil {
+				return err
+			}
+		}
+		c.crypto.signKey, err = cp.GetSignerKeyId(s)
+		if err != nil {
+			return err
+		}
+	}
+	crHeight := 0
+	st := ""
+	switch {
+	case c.sign && c.encrypt:
+		st = fmt.Sprintf("Sign (%s) & Encrypt", c.crypto.signKey)
+		crHeight = 1
+	case c.sign:
+		st = fmt.Sprintf("Sign (%s)", c.crypto.signKey)
+		crHeight = 1
+	case c.encrypt:
+		st = "Encrypt"
+		crHeight = 1
+	default:
+		st = ""
+	}
+	c.crypto.status.Text(st)
+	hHeight := len(c.layout)
+	c.grid.Rows([]ui.GridSpec{
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(hHeight)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(crHeight)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
+	})
+	c.grid.AddChild(c.crypto).At(1, 0)
+	return nil
+}
+
 // Note: this does not reload the editor. You must call this before the first
 // Draw() call.
 func (c *Composer) SetContents(reader io.Reader) *Composer {
@@ -211,6 +347,15 @@ func (c *Composer) AppendContents(reader io.Reader) {
 	c.email.Sync()
 }
 
+func (c *Composer) AppendPart(mimetype string, params map[string]string, body io.Reader) error {
+	if !strings.HasPrefix(mimetype, "text") {
+		return fmt.Errorf("can only append text mimetypes")
+	}
+	c.textParts = append(c.textParts, lib.NewPart(mimetype, params, body))
+	c.resetReview()
+	return nil
+}
+
 func (c *Composer) AddTemplate(template string, data interface{}) error {
 	if template == "" {
 		return nil
@@ -294,20 +439,6 @@ func (c *Composer) FocusTerminal() *Composer {
 	return c
 }
 
-func (c *Composer) FocusSubject() *Composer {
-	c.focusable[c.focused].Focus(false)
-	c.focused = 2
-	c.focusable[c.focused].Focus(true)
-	return c
-}
-
-func (c *Composer) FocusRecipient() *Composer {
-	c.focusable[c.focused].Focus(false)
-	c.focused = 1
-	c.focusable[c.focused].Focus(true)
-	return c
-}
-
 // OnHeaderChange registers an OnChange callback for the specified header.
 func (c *Composer) OnHeaderChange(header string, fn func(subject string)) {
 	if editor, ok := c.editors[strings.ToLower(header)]; ok {
@@ -317,6 +448,15 @@ func (c *Composer) OnHeaderChange(header string, fn func(subject string)) {
 	}
 }
 
+// OnFocusLost registers an OnFocusLost callback for the specified header.
+func (c *Composer) OnFocusLost(header string, fn func(input string) bool) {
+	if editor, ok := c.editors[strings.ToLower(header)]; ok {
+		editor.OnFocusLost(func() {
+			fn(editor.input.String())
+		})
+	}
+}
+
 func (c *Composer) OnClose(fn func(composer *Composer)) {
 	c.onClose = append(c.onClose, fn)
 }
@@ -374,7 +514,7 @@ func (c *Composer) MouseEvent(localX int, localY int, event tcell.Event) {
 	for _, e := range c.focusable {
 		he, ok := e.(*headerEditor)
 		if ok && he.focused {
-			c.FocusEditor(he)
+			c.FocusEditor(he.name)
 		}
 	}
 }
@@ -469,38 +609,29 @@ func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 		var cleartext io.WriteCloser
 		var err error
 
-		var signer *openpgp.Entity
+		signer := ""
 		if c.sign {
-			signer, err = getSigner(c)
-			if err != nil {
-				return err
+			if c.acctConfig.PgpKeyId != "" {
+				signer = c.acctConfig.PgpKeyId
+			} else {
+				signer, err = getSenderEmail(c)
+				if err != nil {
+					return err
+				}
 			}
-		} else {
-			signer = nil
 		}
 
 		if c.encrypt {
-			var to []*openpgp.Entity
 			rcpts, err := getRecipientsEmail(c)
 			if err != nil {
 				return err
 			}
-			for _, rcpt := range rcpts {
-				toEntity, err := lib.GetEntityByEmail(rcpt)
-				if err != nil {
-					return errors.Wrap(err, "no key for "+rcpt)
-				}
-				to = append(to, toEntity)
-			}
-			cleartext, err = pgpmail.Encrypt(&buf, header.Header.Header,
-				to, signer, nil)
-
+			cleartext, err = c.aerc.Crypto.Encrypt(&buf, rcpts, signer, c.aerc.DecryptKeys, header)
 			if err != nil {
 				return err
 			}
 		} else {
-			cleartext, err = pgpmail.Sign(&buf, header.Header.Header,
-				signer, nil)
+			cleartext, err = c.aerc.Crypto.Sign(&buf, signer, c.aerc.DecryptKeys, header)
 			if err != nil {
 				return err
 			}
@@ -510,7 +641,10 @@ func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 		if err != nil {
 			return err
 		}
-		cleartext.Close()
+		err = cleartext.Close()
+		if err != nil {
+			return err
+		}
 		io.Copy(writer, &buf)
 		return nil
 
@@ -520,20 +654,27 @@ func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 }
 
 func writeMsgImpl(c *Composer, header *mail.Header, writer io.Writer) error {
-	if len(c.attachments) == 0 {
-		// no attachements
+	if len(c.attachments) == 0 && len(c.textParts) == 0 {
+		// no attachments
 		return writeInlineBody(header, c.email, writer)
 	} else {
-		// with attachements
+		// with attachments
 		w, err := mail.CreateWriter(writer, *header)
 		if err != nil {
 			return errors.Wrap(err, "CreateWriter")
 		}
-		if err := writeMultipartBody(c.email, w); err != nil {
+		parts := []*lib.Part{
+			lib.NewPart(
+				"text/plain",
+				map[string]string{"Charset": "UTF-8"},
+				c.email,
+			),
+		}
+		if err := writeMultipartBody(append(parts, c.textParts...), w); err != nil {
 			return errors.Wrap(err, "writeMultipartBody")
 		}
 		for _, a := range c.attachments {
-			if err := writeAttachment(a, w); err != nil {
+			if err := a.WriteTo(w); err != nil {
 				return errors.Wrap(err, "writeAttachment")
 			}
 		}
@@ -556,94 +697,52 @@ func writeInlineBody(header *mail.Header, body io.Reader, writer io.Writer) erro
 }
 
 // write the message body to the multipart message
-func writeMultipartBody(body io.Reader, w *mail.Writer) error {
-	bh := mail.InlineHeader{}
-	bh.SetContentType("text/plain", map[string]string{"charset": "UTF-8"})
-
+func writeMultipartBody(parts []*lib.Part, w *mail.Writer) error {
 	bi, err := w.CreateInline()
 	if err != nil {
 		return errors.Wrap(err, "CreateInline")
 	}
 	defer bi.Close()
 
-	bw, err := bi.CreatePart(bh)
-	if err != nil {
-		return errors.Wrap(err, "CreatePart")
-	}
-	defer bw.Close()
-	if _, err := io.Copy(bw, body); err != nil {
-		return errors.Wrap(err, "io.Copy")
-	}
-	return nil
-}
-
-// write the attachment specified by path to the message
-func writeAttachment(path string, writer *mail.Writer) error {
-	f, err := os.Open(path)
-	if err != nil {
-		return errors.Wrap(err, "os.Open")
-	}
-	defer f.Close()
-
-	reader := bufio.NewReader(f)
-
-	// if we have an extension, prefer that instead of trying to sniff the header.
-	// That's generally more accurate than sniffing as lots of things are zip files
-	// under the hood, e.g. most office file types
-	ext := filepath.Ext(path)
-	var mimeString string
-	if mimeString = mime.TypeByExtension(ext); mimeString != "" {
-		// found it in the DB
-	} else {
-		// Sniff the mime type instead
-		// http.DetectContentType only cares about the first 512 bytes
-		head, err := reader.Peek(512)
-		if err != nil && err != io.EOF {
-			return errors.Wrap(err, "Peek")
+	for _, part := range parts {
+		bh := mail.InlineHeader{}
+		bh.SetContentType(part.MimeType, part.Params)
+		bw, err := bi.CreatePart(bh)
+		if err != nil {
+			return errors.Wrap(err, "CreatePart")
+		}
+		defer bw.Close()
+		if _, err := io.Copy(bw, part.Body); err != nil {
+			return errors.Wrap(err, "io.Copy")
 		}
-		mimeString = http.DetectContentType(head)
-	}
-
-	// mimeString can contain type and params (like text encoding),
-	// so we need to break them apart before passing them to the headers
-	mimeType, params, err := mime.ParseMediaType(mimeString)
-	if err != nil {
-		return errors.Wrap(err, "ParseMediaType")
-	}
-	filename := filepath.Base(path)
-	params["name"] = filename
-
-	// set header fields
-	ah := mail.AttachmentHeader{}
-	ah.SetContentType(mimeType, params)
-	// setting the filename auto sets the content disposition
-	ah.SetFilename(filename)
-
-	aw, err := writer.CreateAttachment(ah)
-	if err != nil {
-		return errors.Wrap(err, "CreateAttachment")
-	}
-	defer aw.Close()
-
-	if _, err := reader.WriteTo(aw); err != nil {
-		return errors.Wrap(err, "reader.WriteTo")
 	}
 
 	return nil
 }
 
 func (c *Composer) GetAttachments() []string {
-	return c.attachments
+	var names []string
+	for _, a := range c.attachments {
+		names = append(names, a.Name())
+	}
+	return names
 }
 
 func (c *Composer) AddAttachment(path string) {
-	c.attachments = append(c.attachments, path)
+	c.attachments = append(c.attachments, lib.NewFileAttachment(path))
 	c.resetReview()
 }
 
-func (c *Composer) DeleteAttachment(path string) error {
+func (c *Composer) AddPartAttachment(name string, mimetype string, params map[string]string, body io.Reader) {
+	c.attachments = append(c.attachments, lib.NewPartAttachment(
+		lib.NewPart(mimetype, params, body), name,
+	))
+	c.resetReview()
+}
+
+func (c *Composer) DeleteAttachment(name string) error {
 	for i, a := range c.attachments {
-		if a == path {
+		if a.Name() == name {
 			c.attachments = append(c.attachments[:i], c.attachments[i+1:]...)
 			c.resetReview()
 			return nil
@@ -657,7 +756,7 @@ func (c *Composer) resetReview() {
 	if c.review != nil {
 		c.grid.RemoveChild(c.review)
 		c.review = newReviewMessage(c, nil)
-		c.grid.AddChild(c.review).At(2, 0)
+		c.grid.AddChild(c.review).At(3, 0)
 	}
 }
 
@@ -676,7 +775,7 @@ func (c *Composer) termEvent(event tcell.Event) bool {
 func (c *Composer) termClosed(err error) {
 	c.grid.RemoveChild(c.editor)
 	c.review = newReviewMessage(c, err)
-	c.grid.AddChild(c.review).At(2, 0)
+	c.grid.AddChild(c.review).At(3, 0)
 	c.editor.Destroy()
 	c.editor = nil
 	c.focusable = c.focusable[:len(c.focusable)-1]
@@ -703,7 +802,7 @@ func (c *Composer) ShowTerminal() {
 	c.editor, _ = NewTerminal(editor) // TODO: handle error
 	c.editor.OnEvent = c.termEvent
 	c.editor.OnClose = c.termClosed
-	c.grid.AddChild(c.editor).At(2, 0)
+	c.grid.AddChild(c.editor).At(3, 0)
 	c.focusable = append(c.focusable, c.editor)
 }
 
@@ -722,10 +821,12 @@ func (c *Composer) NextField() {
 	c.focusable[c.focused].Focus(true)
 }
 
-func (c *Composer) FocusEditor(editor *headerEditor) {
+func (c *Composer) FocusEditor(editor string) {
+	editor = strings.ToLower(editor)
 	c.focusable[c.focused].Focus(false)
-	for i, e := range c.focusable {
-		if e == editor {
+	for i, f := range c.focusable {
+		e := f.(*headerEditor)
+		if strings.ToLower(e.name) == editor {
 			c.focused = i
 			break
 		}
@@ -741,11 +842,10 @@ func (c *Composer) AddEditor(header string, value string, appendHeader bool) {
 		e.storeValue() // flush modifications from the user to the header
 		editor = e
 	} else {
-		e := newHeaderEditor(header, c.header,
-			c.aerc.SelectedAccount().UiConfig())
-		if c.config.Ui.CompletionPopovers {
-			e.input.TabComplete(c.completer.ForHeader(header),
-				c.config.Ui.CompletionDelay)
+		uiConfig := c.acct.UiConfig()
+		e := newHeaderEditor(header, c.header, uiConfig)
+		if uiConfig.CompletionPopovers {
+			e.input.TabComplete(c.completer.ForHeader(header), uiConfig.CompletionDelay)
 		}
 		c.editors[header] = e
 		c.layout = append(c.layout, []string{header})
@@ -769,7 +869,7 @@ func (c *Composer) AddEditor(header string, value string, appendHeader bool) {
 		editor.storeValue()
 	}
 	if value == "" {
-		c.FocusEditor(c.editors[header])
+		c.FocusEditor(c.editors[header].name)
 	}
 	c.updateGrid()
 }
@@ -784,24 +884,28 @@ func (c *Composer) updateGrid() {
 
 	if c.grid == nil {
 		c.grid = ui.NewGrid().Columns([]ui.GridSpec{
-			{ui.SIZE_WEIGHT, ui.Const(1)},
+			{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 		})
 	}
-
+	crHeight := 0
+	if c.sign || c.encrypt {
+		crHeight = 1
+	}
 	c.grid.Rows([]ui.GridSpec{
-		{ui.SIZE_EXACT, ui.Const(height)},
-		{ui.SIZE_EXACT, ui.Const(1)},
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(height)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(crHeight)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 
 	if c.heditors != nil {
 		c.grid.RemoveChild(c.heditors)
 	}
-	borderStyle := c.config.Ui.GetStyle(config.STYLE_BORDER)
-	borderChar := c.config.Ui.BorderCharHorizontal
+	borderStyle := c.acct.UiConfig().GetStyle(config.STYLE_BORDER)
+	borderChar := c.acct.UiConfig().BorderCharHorizontal
 	c.heditors = heditors
 	c.grid.AddChild(c.heditors).At(0, 0)
-	c.grid.AddChild(ui.NewFill(borderChar, borderStyle)).At(1, 0)
+	c.grid.AddChild(ui.NewFill(borderChar, borderStyle)).At(2, 0)
 }
 
 func (c *Composer) reloadEmail() error {
@@ -820,11 +924,11 @@ type headerEditor struct {
 	header   *mail.Header
 	focused  bool
 	input    *ui.TextInput
-	uiConfig config.UIConfig
+	uiConfig *config.UIConfig
 }
 
 func newHeaderEditor(name string, h *mail.Header,
-	uiConfig config.UIConfig) *headerEditor {
+	uiConfig *config.UIConfig) *headerEditor {
 	he := &headerEditor{
 		input:    ui.NewTextInput("", uiConfig),
 		name:     name,
@@ -869,6 +973,11 @@ func (he *headerEditor) storeValue() {
 	val := he.input.String()
 	switch strings.ToLower(he.name) {
 	case "to", "from", "cc", "bcc":
+		if strings.TrimSpace(val) == "" {
+			// if header is empty, delete it
+			he.header.Del(he.name)
+			return
+		}
 		list, err := mail.ParseAddressList(val)
 		if err == nil {
 			he.header.SetAddressList(he.name, list)
@@ -877,7 +986,6 @@ func (he *headerEditor) storeValue() {
 			// fix the issue
 			he.header.SetText(he.name, val)
 		}
-		val = format.FormatAddresses(list)
 	default:
 		he.header.SetText(he.name, val)
 	}
@@ -941,6 +1049,12 @@ func (he *headerEditor) OnChange(fn func()) {
 	})
 }
 
+func (he *headerEditor) OnFocusLost(fn func()) {
+	he.input.OnFocusLost(func(_ *ui.TextInput) {
+		fn()
+	})
+}
+
 type reviewMessage struct {
 	composer *Composer
 	grid     *ui.Grid
@@ -974,31 +1088,36 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 		for _, input := range bindings.GetReverseBindings(strokes) {
 			inputs = append(inputs, config.FormatKeyStrokes(input))
 		}
-		if len(inputs) == 0 {
-			continue
-		}
-		actions = append(actions, fmt.Sprintf("  %-6s  %s", strings.Join(inputs[:], ", "), name))
+		actions = append(actions, fmt.Sprintf("  %-6s  %-40s  %s",
+			strings.Join(inputs[:], ", "), name, cmd))
 	}
 
 	spec := []ui.GridSpec{
-		{ui.SIZE_EXACT, ui.Const(1)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)},
 	}
 	for i := 0; i < len(actions)-1; i++ {
-		spec = append(spec, ui.GridSpec{ui.SIZE_EXACT, ui.Const(1)})
+		spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
 	}
-	spec = append(spec, ui.GridSpec{ui.SIZE_EXACT, ui.Const(2)})
-	spec = append(spec, ui.GridSpec{ui.SIZE_EXACT, ui.Const(1)})
+	spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(2)})
+	spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
 	for i := 0; i < len(composer.attachments)-1; i++ {
-		spec = append(spec, ui.GridSpec{ui.SIZE_EXACT, ui.Const(1)})
+		spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
+	}
+	if len(composer.textParts) > 0 {
+		spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
+		spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
+		for i := 0; i < len(composer.textParts); i++ {
+			spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
+		}
 	}
 	// make the last element fill remaining space
-	spec = append(spec, ui.GridSpec{ui.SIZE_WEIGHT, ui.Const(1)})
+	spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)})
 
 	grid := ui.NewGrid().Rows(spec).Columns([]ui.GridSpec{
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 
-	uiConfig := composer.config.Ui
+	uiConfig := composer.acct.UiConfig()
 
 	if err != nil {
 		grid.AddChild(ui.NewText(err.Error(), uiConfig.GetStyle(config.STYLE_ERROR)))
@@ -1019,13 +1138,26 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 		if len(composer.attachments) == 0 {
 			grid.AddChild(ui.NewText("(none)",
 				uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
+			i += 1
 		} else {
 			for _, a := range composer.attachments {
-				grid.AddChild(ui.NewText(a, uiConfig.GetStyle(config.STYLE_DEFAULT))).
+				grid.AddChild(ui.NewText(a.Name(), uiConfig.GetStyle(config.STYLE_DEFAULT))).
 					At(i, 0)
 				i += 1
 			}
 		}
+		if len(composer.textParts) > 0 {
+			grid.AddChild(ui.NewText("Parts:",
+				uiConfig.GetStyle(config.STYLE_TITLE))).At(i, 0)
+			i += 1
+			grid.AddChild(ui.NewText("text/plain", uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
+			i += 1
+			for _, p := range composer.textParts {
+				grid.AddChild(ui.NewText(p.MimeType, uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
+				i += 1
+			}
+
+		}
 	}
 
 	return &reviewMessage{
@@ -1048,29 +1180,71 @@ func (rm *reviewMessage) Draw(ctx *ui.Context) {
 	rm.grid.Draw(ctx)
 }
 
-func getSigner(c *Composer) (signer *openpgp.Entity, err error) {
-	signerEmail, err := getSenderEmail(c)
-	if err != nil {
-		return nil, err
+type cryptoStatus struct {
+	title         string
+	status        *ui.Text
+	uiConfig      *config.UIConfig
+	signKey       string
+	setEncOneShot bool
+}
+
+func newCryptoStatus(uiConfig *config.UIConfig) *cryptoStatus {
+	defaultStyle := uiConfig.GetStyle(config.STYLE_DEFAULT)
+	return &cryptoStatus{
+		title:         "Security",
+		status:        ui.NewText("", defaultStyle),
+		uiConfig:      uiConfig,
+		signKey:       "",
+		setEncOneShot: true,
 	}
-	signer, err = lib.GetSignerEntityByEmail(signerEmail)
-	if err != nil {
-		return nil, err
-	}
+}
+
+func (cs *cryptoStatus) Draw(ctx *ui.Context) {
+	// Extra character to put a blank cell between the header and the input
+	size := runewidth.StringWidth(cs.title+":") + 1
+	defaultStyle := cs.uiConfig.GetStyle(config.STYLE_DEFAULT)
+	titleStyle := cs.uiConfig.GetStyle(config.STYLE_HEADER)
+	ctx.Fill(0, 0, size, ctx.Height(), ' ', defaultStyle)
+	ctx.Printf(0, 0, titleStyle, "%s:", cs.title)
+	cs.status.Draw(ctx.Subcontext(size, 0, ctx.Width()-size, 1))
+}
 
-	key, ok := signer.SigningKey(time.Now())
-	if !ok {
-		return nil, fmt.Errorf("no signing key found for %s", signerEmail)
-	}
+func (cs *cryptoStatus) Invalidate() {
+	cs.status.Invalidate()
+}
 
-	if !key.PrivateKey.Encrypted {
-		return signer, nil
-	}
+func (cs *cryptoStatus) OnInvalidate(fn func(ui.Drawable)) {
+	cs.status.OnInvalidate(func(_ ui.Drawable) {
+		fn(cs)
+	})
+}
 
-	_, err = c.aerc.DecryptKeys([]openpgp.Key{key}, false)
+func (c *Composer) checkEncryptionKeys(_ string) bool {
+	rcpts, err := getRecipientsEmail(c)
 	if err != nil {
-		return nil, err
+		// checkEncryptionKeys gets registered as a callback and must
+		// explicitly call c.SetEncrypt(false) when encryption is not possible
+		c.SetEncrypt(false)
+		st := fmt.Sprintf("Cannot encrypt: %v", err)
+		c.aerc.statusline.PushError(st)
+		return false
+	}
+	var mk []string
+	for _, rcpt := range rcpts {
+		key, err := c.aerc.Crypto.GetKeyId(rcpt)
+		if err != nil || key == "" {
+			mk = append(mk, rcpt)
+		}
 	}
-
-	return signer, nil
+	if len(mk) > 0 {
+		c.SetEncrypt(false)
+		st := fmt.Sprintf("Cannot encrypt, missing keys: %s", strings.Join(mk, ", "))
+		c.aerc.statusline.PushError(st)
+		return false
+	}
+	// If callbacks were registered, encrypt will be set when user removes
+	// recipients with missing keys
+	c.encrypt = true
+	c.updateCrypto()
+	return true
 }
